"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _deasync = require("deasync");

var _deasync2 = _interopRequireDefault(_deasync);

var _less = require("less/lib/less/");

var _less2 = _interopRequireDefault(_less);

var _fileManager = require("less/lib/less-node/file-manager");

var _fileManager2 = _interopRequireDefault(_fileManager);

var _utils = require("less/lib/less/utils");

var _utils2 = _interopRequireDefault(_utils);

var _Variable = require("../tree/Variable");

var _Variable2 = _interopRequireDefault(_Variable);

var _Condition = require("../tree/Condition");

var _Condition2 = _interopRequireDefault(_Condition);

var _Negative = require("../tree/Negative");

var _Negative2 = _interopRequireDefault(_Negative);

var _Dimension = require("../tree/Dimension");

var _Dimension2 = _interopRequireDefault(_Dimension);

var _functions = require("../functions");

var functions = _interopRequireWildcard(_functions);

var _utils3 = require("./utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var transpile = function transpile(less, source, filename) {
    var opts = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

    var banner = "";
    if (opts.import) {
        banner += "@import (reference) \"" + opts.import + "\";";
    }
    var parse = (0, _deasync2.default)(function (input, options, callback) {
        return less.parse(input, options, function (e, root, imports, options) {
            return callback(e, { root: root, imports: imports, options: options });
        });
    });

    var _parse = parse(source, { math: 0, paths: [_path2.default.dirname(filename), process.cwd()], banner: banner }),
        root = _parse.root,
        imports = _parse.imports,
        options = _parse.options;

    if (!root) {
        console.error("Failed to parse", source);
        return source;
    }
    // Disables the validation: Properties must be inside selector blocks. They cannot be in the root
    root.firstRoot = false;
    var parseTree = new less.ParseTree(root, imports);

    var _parseTree$toCSS = parseTree.toCSS(options),
        css = _parseTree$toCSS.css;

    return css;
};

exports.default = function (source, filename, opts) {
    var less = new _less2.default(undefined, [new _fileManager2.default()]);
    var oldGenCSS = less.tree.Expression.prototype.genCSS;
    var oldEval = less.tree.Anonymous.prototype.eval;
    var oldTree = Object.assign({}, less.tree);
    var oldFunctionRegistry = Object.assign({}, less.functions.functionRegistry._data);
    var oldJoinSelector = less.tree.Ruleset.prototype.joinSelector;
    try {
        Object.defineProperty(less.tree.Node.prototype, 'parse', (0, _utils3.nodeParse)(less));
        less.tree.Variable = _Variable2.default;
        less.tree.Condition = _Condition2.default;
        less.tree.Negative = _Negative2.default;
        less.tree.Dimension = _Dimension2.default;
        less.tree.Expression.prototype.genCSS = (0, _utils3.genCSS)(less);
        less.tree.Anonymous.prototype.eval = (0, _utils3.anonymousEval)(less);

        // Changes the function joinSelector to allow & selectors in the root element. Useful for overriding styles with higher specificity.
        var _less$tree = less.tree,
            Paren = _less$tree.Paren,
            Selector = _less$tree.Selector,
            Element = _less$tree.Element;

        var joinSelector = less.tree.Ruleset.prototype.joinSelector.toString().replace("if (el.value !== '&') {", "if (el.value !== '&' || context.length === 0) {");
        eval("less.tree.Ruleset.prototype.joinSelector = " + joinSelector);

        less.functions.functionRegistry.addMultiple(functions);
        less.PluginLoader = function PluginLoader() {
            _classCallCheck(this, PluginLoader);
        };

        return transpile(less, source, filename, opts);
    } finally {
        delete less.tree.Node.prototype.parse;
        Object.assign(less.tree, oldTree);
        less.tree.Expression.prototype.genCSS = oldGenCSS;
        less.tree.Anonymous.prototype.eval = oldEval;
        less.tree.Ruleset.prototype.joinSelector = oldJoinSelector;
        less.functions.functionRegistry._data = oldFunctionRegistry;
    }
};