"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.nodeParse = exports.anonymousEval = exports.genCSS = undefined;

var _VariableNode = require("../tree/VariableNode");

var _VariableNode2 = _interopRequireDefault(_VariableNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var consumeBrackets = function consumeBrackets(parserInput) {
    var inComment = false;
    var blockDepth = 0;
    var blockStack = [];
    var input = parserInput.getInput();
    var length = input.length;
    var lastPos = parserInput.i;
    var i = parserInput.i;
    var prevChar = void 0,
        nextChar = void 0;

    do {
        prevChar, nextChar = input.charAt(i);
        if (blockDepth === 0 && prevChar === "}") {
            return input.substr(lastPos, i - lastPos);
        } else {
            if (inComment) {
                if (nextChar === '*' && input.charAt(i + 1) === '/') {
                    i++;
                    blockDepth--;
                    inComment = false;
                }
                i++;
                continue;
            }
            switch (nextChar) {
                case '/':
                    if (input.charAt(i + 1) === '*') {
                        i++;
                        inComment = true;
                        blockDepth++;
                    }
                    break;
                case "'":
                case '"':
                    var quote = parserInput.$quoted(i);
                    if (quote) {
                        i += quote[1].length - 1;
                    }
                    break;
                case '{':
                    blockStack.push('}');
                    blockDepth++;
                    break;
                case '(':
                    blockStack.push(')');
                    blockDepth++;
                    break;
                case '[':
                    blockStack.push(']');
                    blockDepth++;
                    break;
                case '}':
                case ')':
                case ']':
                    var expected = blockStack.pop();
                    if (nextChar === expected) {
                        blockDepth--;
                    } else {
                        return expected;
                    }
            }
            i++;
        }
        prevChar = nextChar;
    } while (i <= length);
};

var genCSS = exports.genCSS = function genCSS(less) {
    return function (context, output) {
        var cssFragment = context && context.cssFragment && this.value.some(function (v) {
            return v instanceof _VariableNode2.default;
        });
        if (cssFragment) output.add('`/*styless*/');
        for (var i = 0; i < this.value.length; i++) {
            var value = this.value[i];
            if (cssFragment && value instanceof _VariableNode2.default) {
                output.add("${" + value.toCSS() + "}");
            } else {
                value.genCSS(context, output);
            }
            if (!(this.noSpacing || value.noSpacing) && i + 1 < this.value.length) {
                output.add(' ');
            }
        }
        if (cssFragment) output.add('`');
    };
};
var anonymousEval = exports.anonymousEval = function anonymousEval(less) {
    return function () {
        var anonymous = new less.tree.Anonymous(this.value, this._index, this._fileInfo, this.mapLines, this.rulesetLike, this.visibilityInfo());
        anonymous.noSpacing = this.noSpacing;
        return anonymous;
    };
};

var lastSelf = void 0;
var nodeParse = exports.nodeParse = function nodeParse(less) {
    return {
        get: function get() {
            return lastSelf;
        },
        set: function set(self) {
            lastSelf = self;
            var parseJS = function parseJS(orig, treeConstructor, isDeclaration) {
                var val = orig();
                if (val) return val;

                var name = void 0;
                self.parserInput.save();
                if (self.parserInput.currentChar() === '$' && (name = consumeBrackets(self.parserInput))) {
                    // Makes the assumption that all nested code blocks have css in them, or ends with a ;.
                    if (isDeclaration && !name.includes("css")) {
                        self.parserInput.save();
                        self.parserInput.$str(name);
                        if (self.parserInput.currentChar() !== ';') {
                            self.parserInput.restore();
                            return;
                        }
                    }

                    self.parserInput.$str(name);
                    var anonymous = new less.tree.Anonymous(name, self.parserInput.i, self.fileInfo);
                    anonymous.noSpacing = self.parserInput.prevChar() !== " ";
                    if (treeConstructor) {
                        return treeConstructor(anonymous);
                    } else {
                        return anonymous;
                    }
                }
                self.parserInput.restore();
            };

            self.parsers.declaration = parseJS.bind(null, self.parsers.declaration.bind(self.parsers), undefined, true);
            self.parsers.entities.variable = parseJS.bind(null, self.parsers.entities.variable.bind(self.parsers));
            self.parsers.element = parseJS.bind(null, self.parsers.element.bind(self.parsers), function (anonymous) {
                var c = self.parsers.combinator();
                return new less.tree.Element(c, anonymous, true, self.parserInput.i, self.fileInfo);
            });
        },
        configurable: true
    };
};